<?php
/**
* WPPIZZA_DB Class
*
* @package     WPPIZZA
* @subpackage  WPPIZZA_DB
* @copyright   Copyright (c) 2015, Oliver Bach
* @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
* @since       3.0
*
*/
if ( ! defined( 'ABSPATH' ) ) exit;/*Exit if accessed directly*/


/************************************************************************************************************************
*
*
*	WPPIZZA_DB
*
*
************************************************************************************************************************/
class WPPIZZA_DB{


	function __construct() {
		//global $blog_id;
		/* on orderpage, initialize/save session data to db - sanitized */
		add_action('wp', array( $this, 'order_initialize'));
	}


	/**********************************************
		get the most recent completed order of current blog
		used for templates preview
	**********************************************/
	function get_last_completed_blog_order_id($user_id = null){
		global $wpdb;
		$last_order_id = $wpdb->get_row("SELECT MAX(id) as id FROM ".$wpdb->prefix . WPPIZZA_TABLE_ORDERS." WHERE payment_status='COMPLETED' ");
		$order = $this->fetch_order_details(false, $last_order_id->id, false, false, $user_id);/* set $user_id to false to not restrict by logged in user */
		if(!empty($order)){
			$order = $order['orders'][0];
		}
	return $order;
	}

	/**************************************************************************************************

		update order

	**************************************************************************************************/
	function update_order($blog_id, $order_id, $hash, $update_values, $where_payment_status = false) {
		global $wpdb;

		/**
			order table
		**/
		$order_table = $this->order_table($blog_id);

		/**
			where | orderid/hash | where_format
		**/
		$where = array();
		$where_format = array();
		if($order_id){
			$where['id'] = $order_id;
			$where_format[] = '%d';
		}
		if($hash){
			$where['hash'] = $hash;
			$where_format[] = '%s';
		}
		/**
			where | payment_status | where_format
		**/
		if(!empty($where_payment_status)){
			$where['payment_status'] = ''.$where_payment_status.'';
			$where_format[] = '%s';
		}

		/*
			set update vars
		*/
		$order = array();
		$order['data'] = array();
		$order['type'] = array();
		foreach($update_values as $key=>$val){
			$order['data'][$key] = $val['data'];
			$order['type'][] = $val['type'];
		}

		/*
			update order
		*/
		$update_order = $wpdb->update( $order_table , $order['data'], $where , $order['type'], $where_format);
		/*
			return bool db updated or not
		*/
		$update_order = !empty($update_order) ? true : false;

	return $update_order;
	}


	/***************************************************************************************************
		get right table depending on blog id
	***************************************************************************************************/
	function order_table($set_blog_id = false){
		global $wpdb, $blog_id;
		$order_table = $wpdb->prefix;
		if($set_blog_id && $set_blog_id != $blog_id && $set_blog_id>1){
			$order_table .= $set_blog_id.'_';
		}
		$order_table .= WPPIZZA_TABLE_ORDERS;

	return $order_table;
	}

	/**************************************************************************************************
	*
	*
	*	cancel order
	*
	*
	**************************************************************************************************/
	function cancel_order($blog_id, $order_id, $hash, $limit_days = true) {
		global $wpdb;

		/**
			order table
		**/
		$order_table = $this->order_table($blog_id);


		$where = array();
		$where_format = array();
		/**
			where | orderid/hash | where_format
		**/
		if(!empty($order_id)){
			$where['id'] = array('clause'=> '=' , 'value' => (int)$order_id);
		}
		if(!empty($hash)){
			$hash = (string)sanitize_key($hash);
			$where['hash'] = array('clause'=> '=' , 'value' => "'".$hash."'");
		}

		/**
			where | payment_status
			return true for already/previously cancelled orders too
		**/
			$where['payment_status'] = array('clause'=> 'IN' , 'value' => "('INITIALIZED', 'CANCELLED', 'INPROGRESS')");

		/**
			restrict to last 7 days
			unless specifically bypassed
		**/
		if($limit_days){
			$where['order_date'] = array('clause'=> '>' , 'value' => ' TIMESTAMPADD(WEEK,-1,NOW()) ');
		}


		/**
			columns to update
		**/
		$update_values = array();
		/** amend order update */
		$update_values['order_update'] 		= array('data' =>date('Y-m-d H:i:s', WPPIZZA_WP_TIME));
		/* set status, cancelled */
		$update_values['payment_status'] 	= array('data' => 'CANCELLED');


		/*
			set update vars for query
		*/
		$data= array();
		foreach($update_values as $key=>$val){
			$data[$key] = "".$key." = '".$val['data']."'";
		}
		$data = implode(', ', $data);

		/*
			set where clause for query
		*/
		$where_clause= array();
		foreach($where as $key=>$val){
			$where_clause[$key] = "" . $key . ' ' . $val['clause'] . ' ' . $val['value'] . "";
		}
		$where_clause = implode(' AND ', $where_clause);


		/*
			run query
			we cannot use $wpdb->update with IN or > in where clause :(
		*/
		$sql = 'UPDATE '.$order_table.' SET '.$data.' WHERE '.$where_clause.'';
		$update_order = $wpdb->query($sql);
		$update_order = empty($update_order) ? false : true ;/* because we can */


	return $update_order;
	}

	/**************************************************************************************************
	*
	*
	*	delete order
	*
	*
	**************************************************************************************************/
	function delete_order($delete_id, $blog_id = false) {
		global $wpdb;

		$order_table = $this->order_table($blog_id);

		$wpdb->delete( $order_table, array( 'id' => $delete_id ), array( '%d' ) );
	}

	/**************************************************************************************************
	*
	*
	*	initialize order
	* 	insert session into db when coming to orderpage
	*	updates too if adding tips for example
	*
	**************************************************************************************************/
	/*
		insert session into db when coming to orderpage
	*/

	function order_initialize() {
		global $wpdb, $blog_id, $post;
		/* for the time being, set this to false */
		$is_ajax = ( defined('DOING_AJAX') && DOING_AJAX ) ? true : false;

		/**
			check if an orderpage widget is on page ,
			to override check for is_orderpage
			initialize as false
		**/
		$has_orderpage_widget = wppizza_has_orderpage_widget();

		/*
			check is_orderpage
			if it is not an ajax request (to do perhaps, at the moment its always false )
			and do not insert or update
			if we cannot checkout yet anyway

		*/
		if(!$is_ajax && !wppizza_is_orderpage() && !$has_orderpage_widget){
			return;
		}

		/*
			get userdata session to either update or insert order
		*/
		$user_session = WPPIZZA()->session->get_userdata();

		/*
			get mapped order data, generating new hash
		*/
		$order_session = $this->map_order($user_session);


		/*
			which table (blog) should we be using ?
		*/
		$order_table = $this->order_table($blog_id);


		/* check if theres an initialized order already with that hash and update instead of insert new*/
		$update_id = false;
		$insert_id = false;
		if(!empty($user_session[''.WPPIZZA_SLUG.'_hash'])){
			$get_order = $wpdb->get_row( "SELECT id FROM ".$order_table." WHERE hash = '".$user_session[''.WPPIZZA_SLUG.'_hash']."' AND payment_status = 'INITIALIZED' " );
			/* update if theres an order in session */
			if (!empty($order_session) && null !== $get_order ) {
				$update_id = $get_order->id;
			}
		}

		/*
			update db
		*/
		if($update_id){
			$is_update = $wpdb->update( $order_table , $order_session['data'], array( 'id' => $update_id ), $order_session['type'], array( '%d' ));
			/* something failed, let's do a new one to be safe*/
			if(false === $is_update){
				$update_id = false;
			}
		}

		/*
			insert new into db if not update or delete and there's an order in session
		*/
		if(!$update_id){//&& !$delete_id && !empty($order_session)

			/* 
				add hash to db and session when inserting new 
			*/
			$wppizza_hash = wppizza_mkHash($order_session);
			$order_session['data']['hash'] = $wppizza_hash;
			$order_session['type'][] = '%s';

			WPPIZZA()->session->set_order_hash($wppizza_hash);
			$user_session = WPPIZZA()->session->get_userdata();
			$order_session['data']['customer_ini'] = maybe_serialize($user_session);
			$order_session['type'][] = '%s';

			$wpdb->insert( $order_table , $order_session['data'], $order_session['type']);
			$insert_id = $wpdb->insert_id;

			/* 
				add order id to user session when inserting new 
				needed for gateways that use overlay and need to pass on the order id
			*/			
			WPPIZZA()->session->set_order_id($insert_id);
			$order_id = $insert_id;
		}

	return;
	}

	/**************************************************************************************************************************************************************************************
	*
	*
	*
	*	admin get customers by search or id
	*
	*
	*
	**************************************************************************************************************************************************************************************/
	function get_customers($selected_user_id = false, $set_limit = 10 ){
		global $wppizza_options, $wpdb, $blog_id;

		/*ini return array*/
		$customers=array();
		$customers['customers_on_page']=array();
		$customers['total_number_of_customers']=0;
		$customers['results_set']=array();

		/*for consistancy add it here*/
		$sql_payment_status = "'".implode("','",explode(",",WPPIZZA_PAYMENT_STATUS_SUCCESS))."'";

		/*pagination and sliceoffset if set*/
		$paged=0;
		if(!empty($_GET['paged']) && is_numeric($_GET['paged'])){
			$paged=(int)$_GET['paged']-1;
		}

		/**
			search for customers by $_GET['s'] or $_GET['uid']
		**/
		/* if we have a distinctly set user id, set _GET['s'] variable accordingly */
		if(!empty($selected_user_id)){
			$_GET['s'] = (int)$selected_user_id;	
		}
		if(!empty($_GET['s']) || !empty($_GET['uid'])){
			$is_customer_search = true;
		}

		/**
			[restrict by user id > 0]
			will be replaced with wp_user_id IN(1,2,3)
			if search applies
		**/
		$user_id_restrict=' > 0 ';

		/*************************************
			getting orders from all subsites
			only multisite->parent site and only if enabled
		*************************************/
		$multisite_all_orders = apply_filters('wppizza_filter_order_history_all_sites',false);

		/*********************************************************************************************

			getting order tables to query

		********************************************************************************************/
			$blog_tables = array();
			$k=0;
			/* all blogs */
			if ($multisite_all_orders){
	 	   		/*get all and loop through blogs*/
	 	   		$blogs = $wpdb->get_results("SELECT blog_id FROM ".$wpdb->blogs."", ARRAY_A);
				if ($blogs) {
		        	foreach($blogs as $blog) {
		        		switch_to_blog($blog['blog_id']);
		        			/*make sure plugin is active*/
		        			if(is_plugin_active(WPPIZZA_PLUGIN_INDEX)){
								$blog_tables[$blog['blog_id']] = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;
		        			$k++;
		        			}
						restore_current_blog();
		        	}
				}
			}
			/* curent blog only */
			if (!$multisite_all_orders){
				$blog_tables[$blog_id] = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;
			}

		/*****************************************************************************
		*
		*	[doing search of some sort, get possible user id's]
		*
		******************************************************************************/
		if(!empty($is_customer_search)){

			/**
				search for customers by $_GET['s']
			**/
			if(!empty($_GET['s'])){

				$search=esc_sql(wppizza_validate_string($_GET['s']));
				/*searching for id*/
				$id_search = (int)$search ;
				$search_int=(is_numeric($search) && !empty($id_search) ) ? $id_search : false;


				/*searching display_name, user_nicename, user email*/
				$search_columns=array();
				if(!$search_int){
					$search_columns[]= "".$wpdb->base_prefix ."users.display_name LIKE '%".$search."%' ";
					$search_columns[]= "".$wpdb->base_prefix ."users.user_nicename LIKE '%".$search."%' ";
					$search_columns[]= "".$wpdb->base_prefix ."users.user_email LIKE '%".$search."%' ";
				}
				/*if numeric, just search id*/
				if($search_int){
					$search_columns[]= "".$wpdb->base_prefix ."users.ID = ".$search_int." ";
				}

				/**construct the query*/
				$customers_query="";
				$customers_query.= " SELECT " . $wpdb->base_prefix . "users.ID" . PHP_EOL;
				$customers_query.= " FROM " . $wpdb->base_prefix . "users " . PHP_EOL;
				$customers_query.= " WHERE ".PHP_EOL." " . implode(''.PHP_EOL.' OR '.PHP_EOL.'', $search_columns) . " " . PHP_EOL;
			}

			/*get results*/
			$customers_search_results = $wpdb->get_results($customers_query);
			/*
				no search results , bail right now
			*/
			if(empty($customers_search_results)){
				return ;
			}


			$customer_search_found_ids=array();
			if(is_array($customers_search_results)){
			foreach($customers_search_results as $vars){
				$customer_search_found_ids[$vars->ID]=$vars->ID;
			}}
			/*overwrite user id search */
			$user_id_restrict =' IN ('.implode(',',$customer_search_found_ids).') ';
		}

		/*****************************************************************************
		*
		*
		*	[get all customer id's from all blogtables with user_registered date etc]
		*	[sort by user_registered DESC]
		*
		*	[only if not search]
		******************************************************************************/


				$colums=array();
				$colums['user_order_by']	= $wpdb->base_prefix.'users.user_registered as user_order_by';
				$colums['user_registered']	= $wpdb->base_prefix.'users.user_registered';
				$colums['user_nicename']	= $wpdb->base_prefix.'users.user_nicename';
				$colums['user_email']		= $wpdb->base_prefix.'users.user_email';
				$colums['display_name']		= $wpdb->base_prefix.'users.display_name';
				$customers_query='';

				/*****************************************************************************
				*
				*	[list all customrs with completed orders]
				*
				******************************************************************************/
				$c=0;
				foreach($blog_tables as $k=>$blog_table){
					//if($c>0){$customers_query.=PHP_EOL.'UNION ALL'.PHP_EOL;}/*union if more than one table */
					if($c>0){$customers_query.=PHP_EOL.'UNION'.PHP_EOL;}/*union if more than one table */
					$customers_query.= ' SELECT ' . implode(',',$colums) . ', ' . $blog_table . '.wp_user_id' . PHP_EOL;
					$customers_query.= ' FROM ' . $blog_table . ' ' . PHP_EOL;
					$customers_query.= ' LEFT JOIN ' . $wpdb->base_prefix . 'users ON ' . $blog_table . '.wp_user_id = ' . $wpdb->base_prefix . 'users.ID ' . PHP_EOL;
					$customers_query.= ' WHERE ' . $blog_table . '.wp_user_id '.$user_id_restrict.' AND payment_status IN ('.$sql_payment_status.') ' . PHP_EOL;
					$customers_query.= ' GROUP BY ' . $blog_table . '.wp_user_id ' . PHP_EOL;
					//$customers_query.= ' LIMIT '.($paged*$set_limit).','.$set_limit.' ' . PHP_EOL;//dont limit as we want to get all*/
				$c++;
				}


				/* add order by clause to end */
				$customers_query.= ' ORDER BY user_order_by DESC ' . PHP_EOL;

				/*get results*/
				$customers_results = $wpdb->get_results($customers_query, ARRAY_A);

				/**sort by user_registered desc**/
				if($customers_results){
					arsort($customers_results);
				}

			/**********************
			*
			*
			*	[total no of customers according to query ]
			*	for pagination
			*
			***********************/
			$customers['total_number_of_customers'] = is_array($customers_results) ? count($customers_results) : 0;


			/**********************
			*
			*
			*	[slice for customers we need to display on page]
			*	[offset and limited
			*
			***********************/
			$slice_offset= $paged * $set_limit;
			$customers_results = array_slice($customers_results, $slice_offset, $set_limit);


			/**********************
			*
			*
			*	[all user id's of customers displayed on page]
			*	[offset and limited
			*
			***********************/
			$ids_on_page_array = wppizza_array_column($customers_results, 'wp_user_id');
			$ids_on_page=implode(',',$ids_on_page_array);

			/**********************
			*
			*
			*	[get values from all tables for each customer displayed on page]
			*
			*
			***********************/
			$customer_values_query='';
			if(count($blog_tables) >1 ){
				$customer_values_query=PHP_EOL."SELECT wp_user_id as wp_user_id, SUM(table_count) as table_count , SUM(table_total_value) as table_total_value, SUM(table_total_items) as table_total_items  " . PHP_EOL;
				$customer_values_query.=" FROM (" . PHP_EOL;
			}
			$c = 0;
			foreach($blog_tables as $k=>$blog_table){
				if($c > 0){
					$customer_values_query.=" UNION ALL " . PHP_EOL;
				}
				$customer_values_query.=" SELECT wp_user_id, COUNT(*) as table_count,  SUM(order_total) as table_total_value, SUM(order_no_of_items) as table_total_items " . PHP_EOL;
				$customer_values_query.=" FROM ".$blog_table."" . PHP_EOL;
				$customer_values_query.=" WHERE wp_user_id IN (".$ids_on_page.") AND payment_status IN (".$sql_payment_status.") " . PHP_EOL;
				$customer_values_query.=" GROUP BY wp_user_id " . PHP_EOL;
			$c++;
			}
			if(count($blog_tables) >1 ){
				$customer_values_query.=" ) tmp" . PHP_EOL;
				$customer_values_query.=" GROUP BY wp_user_id" . PHP_EOL;
			}

			/**get results - however, if there are no orders (and therefore no customers, just return empty array) **/
			$customer_values_results = empty($ids_on_page_array) ? array() : $wpdb->get_results($customer_values_query, OBJECT_K);

			/**add user data to results set resultset on page**/
			$customers['results_set'] = array();
			foreach($customers_results as $k=>$val){
				$uid=$val['wp_user_id'];/*for convenience*/

				$user_meta_name=array();
				$user_meta_name[]=get_user_meta($uid, 'first_name', true);
				$user_meta_name[]=get_user_meta($uid, 'last_name', true);
				

				$customers['results_set'][$uid]['user_registered']			=	$val['user_registered'];
				$customers['results_set'][$uid]['user_email']				=	$val['user_email'];
				$customers['results_set'][$uid]['user_name']				=	trim(implode(' ',$user_meta_name));
				$customers['results_set'][$uid]['user_display_name']		=	$val['display_name'];
				$customers['results_set'][$uid]['user_user_nicename']		=	$val['user_nicename'];
				$customers['results_set'][$uid]['user_orders_order_count']	=	$customer_values_results[$uid]->table_count;
				$customers['results_set'][$uid]['user_orders_total_value']	=	$customer_values_results[$uid]->table_total_value;
				$customers['results_set'][$uid]['user_orders_total_items']	=	$customer_values_results[$uid]->table_total_items;
				$customers['results_set'][$uid]['user_orders_avg_spent']	=	($customer_values_results[$uid]->table_total_value / $customer_values_results[$uid]->table_total_items);
				$customers['results_set'][$uid]['wp_user_id']				=	$uid;
			}


	return $customers;
	}


	/**************************************************************************************************
	*
	*
	*	all orders - limited as required to count,  status, user etc
	*	$limit = false to get count and sum only
	**************************************************************************************************/
	function get_orders_orderhistory($status, $user_id = false, $limit = false, $paged = false , $custom_status = ''){
		global $wpdb, $blog_id, $wppizza_options;


		/** used gateway idents **/
		$results['gateways_idents'] = array();
		$results['value_orders_on_page'] = 0;

		/*************************************
			if uid is set, only return for this user
			0 for unregistered users
			$user_id is already sanitised here
		*************************************/
		$sql_select_from_user='';
		if( $user_id >=0 ){
			$sql_select_from_user=" AND wp_user_id='".$user_id."' ";
		}

		/*************************************
			getting orders from all subsites
			only multisite->parent site and only if enabled
		*************************************/
		$multisite_all_orders = apply_filters('wppizza_filter_order_history_all_sites',false);

		/*************************************
			what PAYMENT status are we returning - cod being for very old legacy only - include refunded too here
		*************************************/
		$sql_in_payment_status=explode(',',WPPIZZA_PAYMENT_STATUS_SUCCESS);
		$sql_in_payment_status[]='REFUNDED';
		/*************************************
			get failed only
		*************************************/
		if($status == 'FAILED'){
			$sql_in_payment_status=array('FAILED');
		}
		$sql_in_payment_status = "'".implode("','",$sql_in_payment_status)."'";


		/*************************************
			get selected order status to show - skip if looking for FAILED
		*************************************/
		$sql_order_status = '';
		$sql_order_status = (!empty($status) && $status != 'FAILED') ? " AND order_status='".$status."' " : "" ;

		/*************************************
			get selected custom status to show if not empty
		*************************************/
		$sql_custom_status = '';
		$sql_custom_status = (!empty($custom_status) && $custom_status!='[null]') ? " AND order_status_user_defined='".$custom_status."' " : "" ;
		$sql_custom_status = (!empty($custom_status) && $custom_status=='[null]') ? " AND (order_status_user_defined='' OR order_status_user_defined IS NULL) " : $sql_custom_status ;/* show all not set */

		/*********************************************************************************************

			getting tables to query
			getting blogoptions at the same time to be able to add those to the respective results set

		********************************************************************************************/
			$blog_tables = array();
			$blog_info = array();
			$blog_options = array();
			$date_format = array();

			$k=0;
			/* all blogs */
			if ($multisite_all_orders){
	 	   		/*get all and loop through blogs*/
	 	   		$blogs = $wpdb->get_results("SELECT blog_id FROM ".$wpdb->blogs."", ARRAY_A);
				if ($blogs) {
		        	foreach($blogs as $blog) {
		        		switch_to_blog($blog['blog_id']);
		        			/*make sure plugin is active*/
		        			if(is_plugin_active(WPPIZZA_PLUGIN_INDEX)){
								$blog_tables[$blog['blog_id']] = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;
								/*bloginfo from blog - multisite*/
								$blog_info[$blog['blog_id']] = get_blog_details($blog['blog_id']);
								/*wppizza options from blog we switched to*/
								$blog_options[$blog['blog_id']] = get_option(WPPIZZA_SLUG);
								/* get date options for that blog */
								$date_format[$blog['blog_id']]= array('date' => get_option('date_format'), 'time' => get_option('time_format'));
		        			$k++;
		        			}
						restore_current_blog();
		        	}
				}
			}
			/* curent blog only */
			if (!$multisite_all_orders){
				$blog_tables[$blog_id] = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;
				/*bloginfo from blog - get_blog_details does not exist in non network setups*/
				$blog_info[$blog_id] = new stdClass();
				$blog_info[$blog_id] -> blog_id = $blog_id ;
				$blog_info[$blog_id] -> site_id = $blog_id ;
				$blog_info[$blog_id] -> blogname = get_bloginfo('name');
				$blog_info[$blog_id] -> siteurl = get_bloginfo('url');
				$blog_info[$blog_id] -> lang_id = get_bloginfo('language');
				/*wppizza options from blog we switched to*/
				$blog_options[$blog_id] = get_option(WPPIZZA_SLUG);
				/* get date options for that blog */
				$date_format[$blog_id]= array('date' => get_option('date_format'), 'time' => get_option('time_format'));

			}

			/************************************************************************************************************
			*
			*
			*	construct and run the queries
			*
			*
			************************************************************************************************************/

			/***************************
			*	query geting count only
			*	, no limit
			***************************/

			$table_query = array();
			$t=0;
			foreach($blog_tables as $blogId => $table){
				$table_query[$t] = "SELECT ";
				$table_query[$t] .= "COUNT(id) as count ";
				$table_query[$t] .= "FROM ";
				$table_query[$t] .= "" . $table . " ";
				$table_query[$t] .= "WHERE ";
				$table_query[$t] .= "payment_status IN (" . $sql_in_payment_status . ") ";
				$table_query[$t] .= "" . $sql_order_status . " ";
				$table_query[$t] .= "" . $sql_custom_status . " ";
				$table_query[$t] .= "" . $sql_select_from_user . " ";
				/* allow filtering - 2nd parameter to identify count query */
				$table_query[$t] = apply_filters('wppizza_filter_orders_query', $table_query[$t], 'count');

			$t++;
			}
			/* simple sum if more than one table */
			if(count($table_query) > 1){
				$query ="SELECT (";
				$query .= implode(' ) + ( ', $table_query );
				$query .=" ) as count " ;
			}else{
				$query = $table_query[0] ;
			}

			/* run the query */
			$results['total_orders'] = $wpdb->get_results($query, ARRAY_A);
			$results['total_orders'] = $results['total_orders'][0]['count'];/* just to simplify a bit */

			/***************************
				query geting results on page
				, limited and sorted by date
			***************************/
			$table_query = array();
			$t=0;
			foreach($blog_tables as $blogId => $table){
				$table_query[$t] = "SELECT ";
				$table_query[$t] .= "*, order_date as date_sort, '".$blogId."' as blog_id ";
				$table_query[$t] .= "FROM ";
				$table_query[$t] .= "" . $table . " ";
				$table_query[$t] .= "WHERE ";
				$table_query[$t] .= "payment_status IN (".$sql_in_payment_status.") ";
				$table_query[$t] .= "".$sql_order_status." ";
				$table_query[$t] .= "".$sql_custom_status." ";
				$table_query[$t] .= "".$sql_select_from_user." ";
				/* allow filtering - 2nd parameter to identify select/limit query */
				$table_query[$t] = apply_filters('wppizza_filter_orders_query', $table_query[$t], 'select');
			$t++;
			}
			/* union all , limit, sort */
			if(count($table_query) > 1){
				$query =""; //SELECT (
				$query .= implode(' UNION ALL ', $table_query );
				$query .=" " ;
			}else{
				$query = $table_query[0] ;
			}
			$query .=" ORDER BY date_sort DESC LIMIT ".(($paged-1) * $limit).", ".$limit." " ;

			/* run the query */
			$orders = $wpdb->get_results($query, ARRAY_A);


			/**
				add date format , blog options,
				 unserialize order_ini, customer_ini
				 format selected parameters for consistency
			*/
			$results['orders'] = array();
			foreach($orders as $key=>$order){
				/* returned from query */
				$order_blog_id = $order['blog_id'];
				/* all order parameters */
				$results['orders'][$key] = $order;
				/** add blog info */
				$results['orders'][$key]['blog_info'] = $blog_info[$order_blog_id];
				/** add blogs date options/format to order */
				$results['orders'][$key]['date_format'] = $date_format[$order_blog_id];
				/** blog_options as they might be different for orders from different blogs */
				//$results['orders'][$key] ->blog_options = $blog_options[$order_blog_id];
				/* uppercase gateway */
				$results['orders'][$key]['initiator'] = strtoupper($order['initiator']);
				/* lowercase order_status */
				$results['orders'][$key]['order_status'] = strtolower($order['order_status']);
				/* lowercase payment_status */
				$results['orders'][$key]['payment_status'] = strtolower($order['payment_status']);
				/** unserialize order ini overwriting serialized data*/
				$results['orders'][$key]['order_ini'] = maybe_unserialize($order['order_ini']);
				/** unserialize customer ini overwriting serialized data */
				$results['orders'][$key]['customer_ini'] = maybe_unserialize($order['customer_ini']);
				/** unserialize user_data overwriting serialized data */
				$results['orders'][$key]['user_data'] = maybe_unserialize($order['user_data']);
				/**create unique order key for this order as multisite installs may have the same id for different orders on different sites*/
				$results['orders'][$key]['uoKey'] = $order['blog_id'].'_'.$order['id'];

				/** purely for convenience, using blog option currency if not set per order **/
				$results['orders'][$key]['currency'] = (!empty($results['orders'][$key]['order_ini']['param']['currency'])) ? $results['orders'][$key]['order_ini']['param']['currency'] : $blog_options[$order_blog_id]['order_settings']['currency_symbol'];


				/**add used gateway ident using key to end up with unique array**/
				$results['gateways_idents'][$order['initiator']] = $order['initiator'];

				/**add to total ordered amount of shown items ON PAGE**/
				$results['value_orders_on_page'] += $order['order_total'];

			}
		return $results;
	}
	/**************************************************************************************************
	*
	*
	*	get order(s) by blog/hash/wp_user_id/session
	*
	*
	**************************************************************************************************/
	function fetch_order_details($set_blog_id = false, $order_id = false, $hash = false , $payment_status = false, $user_id = null, $multisite = false , $maxpp = 10) {
		global $wpdb, $blog_id;

		/* bit overkill, but doesnt hurt:  also restrict query to user id, bypassed for ipn requests and thank you page (dealt with it separately there) */
		if($user_id === null){
			if(is_user_logged_in()){
				$whereclause[] =' wp_user_id = '.get_current_user_id().'';
			}else{
				$whereclause[] =' wp_user_id = 0 ';
			}
		}
		/* getting orders for distinctly set user id  */
		if(is_numeric($user_id)){
			$whereclause[] =' wp_user_id = '.(int)$user_id.'';
		}
		/** in case there's some tampering going on, unless we have specifically set userid to false, make sure there's no result returned */
		if($user_id !== null && !is_numeric($user_id) && $user_id !== false ){
			$whereclause[] =' wp_user_id = -1 ';
		}

		/** restrict to where  by session id */
		if(!is_user_logged_in() && $user_id !== false){
			$whereclause[] ="session_id ='".session_id()."'";
		}
		/* get by order id */
		if($order_id){
			$whereclause[] ='id='.(int)$order_id.'';
		}
		/* get by hash */
		if($hash){
			$whereclause[] ="hash ='".$hash."'";
		}
		/* restrict to paymnt status, if false, get completed only */
		if($payment_status){
			if(is_array($payment_status)){
				$whereclause[] ="payment_status IN ('".implode("','",$payment_status)."') ";
			}else{
				$whereclause[] ="payment_status='".$payment_status."' ";
			}
		}else{
			$whereclause[] ="payment_status='COMPLETED' ";
		}

		/* max per page if quering for multiple */
		$limit_results = '';
		if(is_numeric($user_id)){
			$limit_results .= ' LIMIT ';
			if(!isset($_GET['pg']) || (int)$_GET['pg']<1){
				$limit_results .= ' 0';
			}else{
				$limit_results .=(int)($_GET['pg']-1)*(int)$maxpp;
			}
			$limit_results .= ','.(int)$maxpp.' ';
		}




		$order_table = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;
		/* switching blogs */
		if($set_blog_id && $set_blog_id != $blog_id && $set_blog_id>1){
			switch_to_blog($set_blog_id);

			/*set order table form blog*/
			$order_table = $wpdb->prefix . WPPIZZA_TABLE_ORDERS;

			/**switch and get wppizza options from blog we switched to **/
			$blog_options[$blog_id]=get_option(WPPIZZA_SLUG);

			/**switch and get blog details blog we switched to **/
			$blog_info[$blog_id] = WPPIZZA() -> helpers -> wppizza_blog_details($blog_id);

			/* get date options for that blog */
			$date_format[$blog_id]['date'] = get_option('date_format');
			$date_format[$blog_id]['time'] = get_option('time_format');

			/**restore current**/
			restore_current_blog();
		}else{
			global $wppizza_options;
			$blog_options[$blog_id] = $wppizza_options;
			$blog_info[$blog_id] = WPPIZZA() -> helpers -> wppizza_blog_details($blog_id);
			/* get date options for that blog */
			$date_format[$blog_id]['date'] = get_option('date_format');
			$date_format[$blog_id]['time'] = get_option('time_format');
		}

		/*******************************************************************
			get single row only - when querying for a specific order by id or hash
		*******************************************************************/
		//if(!$user_id)
		if($user_id === null || $user_id === false ){
			/* set query */
			$query = "SELECT * FROM ".$order_table." WHERE ".implode(' AND ', $whereclause)."  ";
			/* run query*/
			$get_orders = $wpdb->get_row($query);
			if ( $get_orders !== null ) {
				/* return even single results as array with key [0] for consistency */
				$orders['orders'][0] = $get_orders;
				$orders['count'] = 1;
			}
		}

		/*******************************************************************
			get all completed orders for this user - order history
			limited to max per page and from all subsites if enabled and multisite
		*******************************************************************/
		//if($user_id)
		if($user_id !== null && $user_id !== false ){

			/**multisite getting all orders from all sites*/
			if(is_multisite() && $multisite ){

		print"----------".__CLASS__."--- multsite query TODO WHEN USING MULTISITE-------(i think that's done now using get_orders function above ???)-----------";
		exit();

			}

			/**not multisite or not getting all orders from all sites*/
			if(!is_multisite() || !$multisite ){
				/* set query */
				$query = "SELECT * FROM ".$order_table." WHERE ".implode(' AND ', $whereclause)."  ORDER BY order_date DESC ".$limit_results."";
				/* run query */
				$get_orders = $wpdb->get_results($query);
			}

			/*
				run query
			*/


			if ( $get_orders !== null ) {
				/*
					now we know there are results, get the count too, no point running it any earlier
				*/
				$orders['orders'] = $get_orders;
				/* run same query without limit and order by */
				$orders['count'] = $wpdb->get_var( "SELECT COUNT(*) FROM  ".$order_table." WHERE ".implode(' AND ', $whereclause)." " );//temp disabled AND payment_status = 'COMPLETED'
			}
		}

		if (isset($orders['count']) && $orders['count']>=1) {
			/** add date format , blog options and unserialize order_ini and customer_ini for each result*/
			foreach($orders['orders'] as $key=>$order){
				/** add blogs date options/format to order */
				$orders['orders'][$key] ->date_format = $date_format[$blog_id];
				/** blog_info (name etc) as they might be different for different blogs */
				$orders['orders'][$key] ->blog_info = $blog_info[$blog_id];
				/** blog_options as they might be different for different blogs */
				$orders['orders'][$key] ->blog_options = $blog_options[$blog_id];
				/** unserialize order ini */
				$orders['orders'][$key] ->order_ini = maybe_unserialize($order->order_ini);
				/** unserialize customer ini */
				$orders['orders'][$key] ->customer_ini = maybe_unserialize($order->customer_ini);
			}
		}else{
			/* return false if order count <=0 */
			$orders = false;
		}

	return $orders;
	}

	/**************************************************************************************************
	*
	*
	*	map session order details to db fields
	*	returns false if we cannot checkout yet
	*
	*
	**************************************************************************************************/
	function map_order($user_session, $checkout_parameters_only = false) {
		global $current_user, $wpdb, $blog_id;
		/*
			grab order details in current session
			, unset irrelevant for storing,
			add some unique id and session id(to make sure it is really unique)
		*/
		$order_session = WPPIZZA()->session->sort_and_calculate_cart(true);
		$order_session_checkout_parameters = $order_session['checkout_parameters'];
		/** only get is_checkout, can_checkout etc - orderpage **/
		if(!empty($checkout_parameters_only)){
			return $order_session_checkout_parameters;
		}

		/* unset unnecessary parameters that are only used when calculating things in pages etc but are not relevant when storing data in db*/
		unset($order_session['checkout_parameters']);


		/* add session id */
		$order_session['info']['session_id'] = session_id();
		/* add unique ident */
		$order_session['info']['unique_id'] = (function_exists('microtime')) ? microtime(true) : time();

		/*
			current date based on WP_time
		*/
		$order_date = date('Y-m-d H:i:s', WPPIZZA_WP_TIME );

		/*
			UTC
		*/
		$order_date_utc = date('Y-m-d H:i:s', WPPIZZA_UTC_TIME );

		/*

			map data

		*/
		$wp_user_id 				= $current_user->ID; /* user id or 0 if not logged in */
		$order_date					= $order_date; /* current time based on WP timezone */
		$order_date_utc				= $order_date_utc; /* utc */
		$order_update				= '0000-00-00 00:00:00';/* 0  until status change, notes added or similar */
		$order_delivered			= '0000-00-00 00:00:00'; /* initialize as 0 when adding to db */
		$customer_details 			= '';
		$order_details				= '';
		$order_status 				= 'NEW';
		$order_ini 					= maybe_serialize(apply_filters('wppizza_filter_add_to_order_ini',$order_session));
		$order_no_of_items 			= $order_session['summary']['number_of_items'];
		$order_items_total 			= $order_session['summary']['total_price_items'];
		$order_discount 			= $order_session['summary']['discount'];
		//$order_taxes 				= empty($order_session['param']['tax_included']) ?  $order_session['summary']['taxes'] : 0 ;
		$order_taxes 				= !empty($order_session['summary']['taxes']) ?  $order_session['summary']['taxes'] : 0 ;
		$order_taxes_included 		= !empty($order_session['param']['tax_included']) ?  'Y' : 'N' ;
		$order_delivery_charges 	= $order_session['summary']['delivery_charges'];
		$order_handling_charges 	= $order_session['summary']['handling_charges'];
		$order_tips 				= $order_session['summary']['tips'];
		$order_self_pickup 			= !empty($order_session['summary']['self_pickup']) ?  'Y' : 'N' ;
		$order_total 				= $order_session['summary']['total'];
		$customer_ini 				= maybe_serialize(apply_filters('wppizza_filter_add_to_customer_ini', $user_session, 'session'));/* allow arbitrary array data to be added/stored  in customer_ini (i.e user session) to - perhaps -	save some additional values without outputting them anywhere by default */
		$payment_status 			= 'INITIALIZED';
		$transaction_id 			= '';
		$transaction_details 		= '';
		$transaction_errors 		= '';
		$display_errors 			= '';

		$validate_initiator 		= wppizza_validate_alpha_only($user_session[''.WPPIZZA_SLUG.'_gateway_selected']);/* php 5.3 */
		$initiator 					= !empty($validate_initiator) ? $validate_initiator : 'COD';

		$mail_sent 					= 'N';
		$mail_error 				= '';
		$notes 						= '';
		$session_id 				= session_id();
		$ip_address 				= !empty($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0';
		$user_defined 				= maybe_serialize(apply_filters('wppizza_db_column_user_defined','', $order_session, $user_session));/* a text field that can be freely used - serialized if necessary*/
		$user_data=array();
			$user_data['HTTP_USER_AGENT']=!empty($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '--n/a--';
			$user_data['REMOTE_ADDR']=!empty($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '--n/a--';
			$user_data['HTTP_REFERER']=!empty($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '--n/a--';
		$user_data = maybe_serialize($user_data);
		/*

			store data in db

		*/
		$order_data = array();
		/** map fields to data, all fields, but autoinserts or irrelevant for first insert commented out */
		/*only inserting or updateing INITIALIZED order */
		$order_data['wp_user_id'] = array('data' => $wp_user_id, 'type' => '%d' );
		$order_data['order_date'] = array('data' => $order_date, 'type' => '%s' );
		$order_data['order_date_utc'] = array('data' => $order_date_utc, 'type' => '%s' );
		$order_data['order_update'] = array('data' => $order_update, 'type' => '%s' );
		$order_data['order_delivered'] = array('data' => $order_delivered, 'type' => '%s' );
		$order_data['customer_details'] = array('data' => $customer_details, 'type' => '%s' );
		$order_data['order_details'] = array('data' => $order_details, 'type' => '%s' );
		$order_data['order_status'] = array('data' => $order_status, 'type' => '%s' );
		$order_data['order_ini'] = array('data' => $order_ini, 'type' => '%s' );
		$order_data['order_no_of_items'] = array('data' => $order_no_of_items, 'type' => '%d' );
		$order_data['order_items_total'] = array('data' => $order_items_total, 'type' => '%f' );
		$order_data['order_discount'] = array('data' => $order_discount, 'type' => '%f' );
		$order_data['order_taxes'] = array('data' => $order_taxes, 'type' => '%f' );
		$order_data['order_taxes_included'] = array('data' => $order_taxes_included, 'type' => '%s' );
		$order_data['order_delivery_charges'] = array('data' => $order_delivery_charges, 'type' => '%f' );
		$order_data['order_handling_charges'] = array('data' => $order_handling_charges, 'type' => '%f' );
		$order_data['order_tips'] = array('data' => $order_tips, 'type' => '%f' );
		$order_data['order_self_pickup'] = array('data' => $order_self_pickup, 'type' => '%s' );
		$order_data['order_total'] = array('data' => $order_total, 'type' => '%s' );
		$order_data['customer_ini'] = array('data' => $customer_ini, 'type' => '%s' );
		$order_data['payment_status'] = array('data' => $payment_status, 'type' => '%s' );
		$order_data['transaction_id'] = array('data' => $transaction_id, 'type' => '%s' );
		$order_data['transaction_details'] = array('data' => $transaction_details, 'type' => '%s' );
		$order_data['transaction_errors'] = array('data' => $transaction_errors, 'type' => '%s' );
		$order_data['display_errors'] = array('data' => $display_errors, 'type' => '%s' );
		$order_data['initiator'] = array('data' => $initiator, 'type' => '%s' );
		$order_data['mail_sent'] = array('data' => $mail_sent, 'type' => '%s' );
		$order_data['mail_error'] = array('data' => $mail_error, 'type' => '%s' );
		$order_data['notes'] = array('data' => $notes, 'type' => '%s' );
		$order_data['session_id'] = array('data' => $session_id, 'type' => '%s' );
		$order_data['ip_address'] = array('data' => $ip_address, 'type' => '%s' );
		$order_data['user_data'] = array('data' => $user_data, 'type' => '%s' );
		$order_data['user_defined'] = array('data' => $user_defined, 'type' => '%s' );


		/**
			added filtering - not used in plugin
			to allow other plugins to add their own data if - for example - they
			have added their own columns (or indeed change what goes in it)
		**/
		$order_data = apply_filters('wppizza_filter_db_column_data', $order_data, $user_session);


		$order = array();
		$order['data'] = array();
		$order['type'] = array();
		foreach($order_data as $key=>$val){
			$order['data'][$key] = $val['data'];
			$order['type'][] = $val['type'];
		}
		/* get session checkout parameters */
		$order['checkout_parameters'] = $order_session_checkout_parameters;

	return $order;
	}
}
?>